/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_capture.c,v 1.2 2007-06-24 21:02:18 jailbird Exp $
 */

#include "stdinc.h"
#include "modules.h"
#include "hook.h"
#include "client.h"
#include "ircd.h"
#include "send.h"
#include "s_conf.h"
#include "s_serv.h"
#include "hash.h"

static int ms_capture(struct Client *client_p, struct Client *source_p, int parc, const char *parv[]);
static int mo_uncapture(struct Client *client_p, struct Client *source_p, int parc, const char *parv[]);

struct Message capture_msgtab = {
  "CAPTURE",            /* the /COMMAND you want */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  MFLG_SLOW,            /* ALWAYS SET TO MFLG_SLOW */
  { mg_ignore, mg_ignore, mg_ignore, mg_ignore, {ms_capture, 2}, mg_ignore, mg_ignore }
};
struct Message uncapture_msgtab = {
  "UNCAPTURE",          /* the /COMMAND you want */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  0,                    /* SET TO ZERO -- number of times command used by clients */
  MFLG_SLOW,            /* ALWAYS SET TO MFLG_SLOW */
  { mg_ignore, mg_ignore, {mo_uncapture, 2}, {mo_uncapture, 2}, {mo_uncapture, 2}, {mo_uncapture, 2}, mg_ignore }
};

mapi_clist_av1 capture_clist[] = { &capture_msgtab, &uncapture_msgtab, NULL };

DECLARE_MODULE_AV1(capture, NULL, NULL, capture_clist, NULL, NULL, "$Revision: 1.2 $");

/*
 * ms_capture
 *      parv[0] = sender prefix
 *      parv[1] = person to capture
 */

static int
ms_capture(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
	struct Client	*target_p;
	int		chasing;

	if(!IsService(source_p))
		return 0;

	if(!(target_p = find_chasing(source_p, parv[1], &chasing)))
		return 0;

	/* Here is one exception to the "S: can do anything" rule. Don't CAPTURE
	 * opers, since they could always uncapture themselves anyways.
	 * -JailBird */
	if (IsOper(target_p))
		return 0;
	if (MyClient(target_p))
		SetCaptured(target_p);
	sendto_realops_flags(UMODE_ALL, L_ALL,
			     "%s has captured connection for [%s!%s@%s%%%s]",
			     source_p->name, 
			     target_p->name, target_p->username,
			     target_p->host, target_p->servptr->name);
	sendto_match_servs(source_p, target_p->servptr->name, CAP_ENCAP, NOCAPS,
			   "ENCAP %s CAPTURE %s",
			   target_p->servptr->name, get_id(target_p, target_p));
	return 0;
}

/* Taken from hybrid-7.2 contrib/ */
/* mo_uncapture
 *      parv[0] = sender prefix
 *      parv[1] = nickname masklist
 */
static int
mo_uncapture(struct Client *client_p, struct Client *source_p,
             int parc, const char *parv[])
{
  struct Client *target_p = NULL;
  char *nick = NULL, *user = NULL, *host = NULL, *p = NULL;
  dlink_node *ptr = NULL;

  if (MyClient(source_p) && !IsAdmin(source_p))
  {
    sendto_one(source_p, form_str(ERR_NOPRIVS),
               me.name, source_p->name, "CAPTURE");
    return 0;
  }

  if (parc < 2 || EmptyString(parv[1]))
  {
    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
               me.name, source_p->name);
    return 0;
  }

  nick = LOCAL_COPY(parv[1]);

  if ((p = strchr(nick, '@')) == NULL)
  {      
    if ((target_p = find_client(nick)) != NULL && IsClient(target_p))
    {
      if (MyConnect(target_p))
      {
        if (IsCaptured(target_p))
        {
          ClearCaptured(target_p);

          sendto_realops_flags(UMODE_ALL, L_ALL, "Uncaptured %s (%s@%s)",
                               target_p->name, target_p->username,
                               target_p->host);
        }

        sendto_one(source_p, form_str(RPL_ISUNCAPTURED),
                   me.name, source_p->name, target_p->name);
      }
      else
		sendto_match_servs(source_p, target_p->servptr->name, CAP_ENCAP, NOCAPS,
				   "ENCAP %s UNCAPTURE %s",
				   target_p->servptr->name, get_id(target_p, target_p));
    }
    else
      sendto_one(source_p, form_str(ERR_NOSUCHNICK),
                 me.name, source_p->name, nick);
  }
  else
  {
    unsigned int matches = 0;

    /* p != NULL so user @ host given */
    *p++ = '\0';
    host = p;

    if ((p = strchr(nick, '!')) != NULL)
    {
      *p++ = '\0';
      user = p;
    }
    else
    {
      user = nick;
      strcpy(nick, "*");
    }

    if (IsClient(client_p))
	sendto_match_servs(source_p, target_p->servptr->name, CAP_ENCAP, NOCAPS,
			   "ENCAP %s UNCAPTURE %s!%s@%s",
			   target_p->servptr->name, nick, user, host);

    DLINK_FOREACH(ptr, lclient_list.head)
    {
      target_p = ptr->data;

      if (!IsCaptured(target_p))
        continue;

      if (match(nick, target_p->name) &&
          match(host, target_p->host) && match(user, target_p->username))
      {
        ClearCaptured(target_p);
        ++matches;
      }
    }

    sendto_realops_flags(UMODE_ALL, L_ALL,
                         "Bulk uncaptured %s!%s@%s, %u local match(es)",
                         nick, user, host, matches);
  }
  return 0;
}
